
#include "utils.h"
/****************************************************************************
**
**  getSysVar (int)
**
**  **avb
**
*************************************/

Acad::ErrorStatus
getSysVar(LPCTSTR varName, int& val)
{
    resbuf rb;
    if (acedGetVar(varName, &rb) == RTNORM) {
        //ASSERT(rb.restype == RTREAL);
        val = rb.resval.rint;
        return(Acad::eOk);
    }
    else
        return(Acad::eInvalidInput);
}
/******************************************************************
**  int getLayerColor(char* nameLayer)
**    
**	  .
**
**  **avb
**
****************************/

int getLayerColor(char* nameLayer)
{
int color;		
AcDbLayerTable *pLayerTbl;
acdbHostApplicationServices()->workingDatabase()
->getSymbolTable(pLayerTbl, AcDb::kForRead);
if (pLayerTbl->has(nameLayer)) 
{
	AcDbLayerTableRecord *pLayerTblRcd;
	pLayerTbl->getAt(nameLayer, pLayerTblRcd,AcDb::kForRead) ;
	pLayerTbl->close();
	AcCmColor clr;
	clr=pLayerTblRcd->color();
	pLayerTblRcd->close();
	color=clr.colorIndex();
}
return color;
}


/******************************************************************
**  static Acad::ErrorStatus
**  getLayerIdFromString(const char* str, AcDbObjectId& id)
**  
**	 Id ,   .
**
**  **avb
**
****************************/

static Acad::ErrorStatus
getLayerIdFromString(const char* str, AcDbObjectId& id)
{
    Acad::ErrorStatus err;
 
    // Get the table of currently loaded layers.
    //
    AcDbLayerTable *pLayerTable;
    err = acdbHostApplicationServices()->workingDatabase()
        ->getSymbolTable(pLayerTable, AcDb::kForRead);
    if (err != Acad::eOk)
        return err;
 
    // Get the ID of the layer with the name that 'str'
    // contains.
    //
    err = pLayerTable->getAt(str, id, Adesk::kTrue);
 
    pLayerTable->close();
 
    return err;
}
/******************************************************************
**  Acad::ErrorStatus
**	postToDatabase(AcDbEntity* pEnt,AcDbObjectId& idObj)
**  
**	   MODEL_SPACE   .
**
**  **avb
**
****************************/

Acad::ErrorStatus
postToDatabase(/*[in]*/AcDbEntity* pEnt,/*[out]*/AcDbObjectId& idObj)
{
int val;
	getSysVar("TILEMODE", val);
	Acad::ErrorStatus	  es;
	AcDbBlockTable*		pBlockTable;
	AcDbBlockTableRecord*  pSpaceRecord;
    if (acdbHostApplicationServices()->workingDatabase()==NULL)
        return Acad::eNoDatabase;
	if (val==1)
	{
    if ((es = acdbHostApplicationServices()->workingDatabase()->getBlockTable(pBlockTable, AcDb::kForRead))==Acad::eOk){
        if ((es = pBlockTable->getAt(ACDB_MODEL_SPACE, pSpaceRecord, AcDb::kForWrite))==Acad::eOk){
            if ((es = pSpaceRecord->appendAcDbEntity(idObj, pEnt))==Acad::eOk)
                pEnt->close();
            pSpaceRecord->close();
        }
	    pBlockTable->close();
    }
	}
	else if (val==0)
	{
    if ((es = acdbHostApplicationServices()->workingDatabase()->getBlockTable(pBlockTable, AcDb::kForRead))==Acad::eOk){
        if ((es = pBlockTable->getAt(ACDB_PAPER_SPACE, pSpaceRecord, AcDb::kForWrite))==Acad::eOk){
            if ((es = pSpaceRecord->appendAcDbEntity(idObj, pEnt))==Acad::eOk)
                pEnt->close();
            pSpaceRecord->close();
        }
	    pBlockTable->close();
    }
	
	}
return es;}

/******************************************************************
**  Acad::ErrorStatus newLine(ads_point pt1, ads_point pt2)
**  
**	 
**
**  **avb
**
****************************/
Acad::ErrorStatus newLine(ads_point pt1, ads_point pt2)
{
	AcDbLine* pLine = new AcDbLine(asPnt3d(pt1), asPnt3d(pt2));
	if (!pLine)
	{
		acedAlert("Not enough memory to create a Line!");
		return Acad::eOutOfMemory;
	}
	AcDbObjectId id;
	return postToDatabase(pLine, id);
}
/******************************************************************
**  void createPolyline()
**  
**	 2d   
**
**  **avb
**
****************************/

void 
create2Polyline(AcGePoint3d sp,	AcGePoint3d ep, double sw,
				int clInd, char* linTyp)
{
	AcGePoint3dArray ptArr;
    ptArr.setLogicalLength(2);
	ptArr[0].set((double)(sp[0]), (double)(sp[1]), 0.0);
	ptArr[1].set((double)(ep[0]), (double)(ep[1]), 0.0);

    AcDb2dPolyline *pNewPline = new AcDb2dPolyline(
    AcDb::k2dSimplePoly, ptArr, 0.0, Adesk::kFalse,	sw,sw);
   	AcDbObjectId id;
	postToDatabase(pNewPline, id);

	pNewPline->close();
	changeColor(id,clInd);
	setTypeLine(id,linTyp);

	return;
}
/******************************************************************
**Acad::ErrorStatus
**changeColor(AcDbObjectId entId, Adesk::UInt16 newColor)
**  
**	 
**
**  **avb
**
****************************/
Acad::ErrorStatus
changeColor(AcDbObjectId entId, Adesk::UInt16 newColor)
{
    AcDbEntity *pEntity;
    acdbOpenObject(pEntity, entId,
        AcDb::kForWrite);
    pEntity->setColorIndex(newColor);
    pEntity->close();

    return Acad::eOk;

}
/******************************************************************
**	int get_ColorIndex(AcDbObjectId entId)
**  
**	  
**
**  **avb
**
****************************/
int get_ColorIndex(AcDbObjectId entId)
{
    AcDbEntity *pEntity;
    acdbOpenObject(pEntity, entId,
        AcDb::kForWrite);
    pEntity->close();
    return pEntity->colorIndex();
}

/******************************************************************
**	Acad::ErrorStatus
**	setTypeLine(AcDbObjectId entId,char* newVal)
**  
**	  
**
**  **avb
**
****************************/
Acad::ErrorStatus
setTypeLine(AcDbObjectId entId,char* newVal)
{
    AcDbEntity *pEntity;
    acdbOpenObject(pEntity, entId,
        AcDb::kForWrite);

    pEntity->setLinetype(newVal,Adesk::kTrue);
    pEntity->close();

    return Acad::eOk;

}
/******************************************************************
**	int getIntSysVar(char* pszVarName)
**  
**	  
**
**  **avb
**
****************************/
int getIntSysVar(char* pszVarName)
{   
    struct resbuf rb;

    acedGetVar(pszVarName,&rb);

    return(rb.resval.rint);
}
/******************************************************************
**	int getChSysVar(char* pszVarName)
**  
**	  
**
**  **avb
**
****************************/
const char* getChSysVar(const char* pszVarName)
{   
    struct resbuf rb;

    acedGetVar(pszVarName,&rb);

    return(rb.resval.rstring);
}
/******************************************************************
**	bool setIntSysVar(char* pszVarName, int val)
**  
**	  
**
**  **avb
**
****************************/
bool setIntSysVar(char* pszVarName, int val)
{   
    struct resbuf rb;

    rb.restype = RTSHORT;
    rb.resval.rint = val;

    return (acedSetVar(pszVarName,&rb) == RTNORM);
}
/******************************************************************
**	bool setDblSysVar(char* pszVarName, double val)
**  
**	  
**
**  **avb
**
****************************/
bool setDblSysVar(char* pszVarName, double val)
{   
    struct resbuf rb;

    rb.restype = RTREAL;
    rb.resval.rreal = val;

    return (acedSetVar(pszVarName,&rb) == RTNORM);
}

/******************************************************************
**	void highlightAll(const AcDbObjectId& objId)
**  
**	 
**
**  **avb
**
****************************/
void highlightAll(const AcDbObjectId& objId)
{
   
    AcDbEntity *pEnt;
    acdbOpenAcDbEntity(pEnt, objId, AcDb::kForRead);
    pEnt->highlight();
    pEnt->close();
	return ;
}
/******************************************************************
**	void unhighlightAll(const AcDbObjectId& objId)
**  
**	  
**
**  **avb
**
****************************/
void unhighlightAll(const AcDbObjectId& objId)
{
    
    AcDbEntity *pEnt;
    acdbOpenAcDbEntity(pEnt, objId, AcDb::kForRead);
    pEnt->unhighlight();
    pEnt->close();
}
/******************************************************************
**	double iterate(AcDbObjectId plineId)
**  
**	    2d
**
**  **avb
**
****************************/
double iterate(AcDbObjectId plineId)
{
    AcDb2dPolyline *pPline;
    acdbOpenObject(pPline, plineId, AcDb::kForRead);
    AcDbObjectIterator *pVertIter= pPline->vertexIterator();
    pPline->close();  
    AcDb2dVertex *pVertex;
    AcGePoint3d lc,plc;
    AcDbObjectId vertexObjId;
	double dist,distall;
	int i=0;
	for (int vertexNumber = 0; !pVertIter->done();
        vertexNumber++, pVertIter->step())
		{ 
        vertexObjId = pVertIter->objectId();
        acdbOpenObject(pVertex, vertexObjId,
            AcDb::kForRead);
        lc = pVertex->position();
        pVertex->close();
        //acutPrintf("\n  #%d"
        //    " : %0.3f, %0.3f, %0.3f", vertexNumber,
        //    lc[X], lc[Y], lc[Z]);
		if (vertexNumber == 0)
		{
		 dist = 0;
		 distall=0;
		}
		else if (vertexNumber != 0)
		{	
		 dist = sqrt((plc[X] - lc[X])*(plc[X] - lc[X])
		 + (plc[Y] - lc[Y])*(plc[Y] - lc[Y]));
		 distall=distall + dist;
		}
					 plc[X] = lc[X];
					 plc[Y] = lc[Y];
		//			 acutPrintf("    = %f\n",  dist );
		//			 acutPrintf("   = %f\n",  distall );
    }
    delete pVertIter;
	//acutPrintf(" \n  = %f\n",  distall );
	return distall;
}
/****************************************************************************
**
**	plineLength
**
**	   
**
**	**jma
**
*************************************/

double plineLength(AcDbObjectId eId)
{
    Acad::ErrorStatus es;
	AcDbObject *pObj;
	AcGePoint3d pt;
	acdbOpenObject(pObj, eId, AcDb::kForRead);
	double dist=0,x0=0, x1=0, x2=0 ,y0=0, y1=0, y2=0;
	AcDbPolyline* pPline = AcDbPolyline::cast(pObj);
	//Adesk::Boolean isClosed() const;
    for (unsigned int i=0; i<pPline->numVerts(); i++) {
	es = pPline->getPointAt(i, pt);
		if (i==0) {
		x0 = pt[0];
		y0 = pt[1];
		x1 = pt[0];
		y1 = pt[1];}
		else {
		x2 = pt[0];
		y2 = pt[1];
		dist = dist + sqrt((x1 - x2) * (x1 - x2) 
			+ (y1 - y2) * (y1 - y2));
		x1=x2;
		y1=y2;
		}
		
	}
	if (pPline->isClosed()==TRUE){
	dist = dist + sqrt((x0 - x2) * (x0 - x2) 
		+ (y0 - y2) * (y0 - y2));}
    pObj->close();
return dist;
}

/****************************************************************************
**
**	void pline2dWidth(const AcDb2dPolyline* pline)
**
**	    2d
**
**	**avb
**
*************************************/

void pline2dWidth(const AcDb2dPolyline* pline, double sw)
{
    

    AcDbObjectIterator* vertexIter = pline->vertexIterator();
    if(vertexIter != NULL){
    if (vertexIter == NULL)
        return ;

    AcDb2dVertex* vertex;
    for (; !vertexIter->done(); vertexIter->step()) {
        if (acdbOpenObject(vertex, vertexIter->objectId(), AcDb::kForWrite) == Acad::eOk) {
            if (vertex->vertexType() != AcDb::k2dSplineCtlVertex) {
                vertex->setStartWidth(sw);
				vertex->setEndWidth(sw);
                    vertex->close();
                
            }
            vertex->close();
        }
    }
    delete vertexIter;
	}
    return ;
}
/****************************************************************************
**
**	ArxDbgUtils::plineWidth
**
**	   
**
**	**jma
**
*************************************/

void plineWidth(AcDbObjectId eId, double sw)
{
    Acad::ErrorStatus es;
	AcDbObject *pObj;
	acdbOpenObject(pObj, eId, AcDb::kForWrite);
	AcDbPolyline* pPline = AcDbPolyline::cast(pObj);
    for (unsigned int i=0; i<pPline->numVerts(); i++) {
	es = pPline->setWidthsAt (i,sw,sw); }
    pObj->close();
return;
}
/******************************************************************
**	double lenLine(const AcDbObjectId& eId)
**  
**	   
**
**  **avb
**
****************************/
double lenLine(const AcDbObjectId& eId)
{
		AcDbLine *pLine;
		AcGePoint3d sp, ep;
		double dist=0, distall=0;
		acdbOpenObject(pLine, eId, AcDb::kForRead);
		pLine->close();
		sp=pLine->startPoint();
		ep=pLine->endPoint();
		dist = sqrt((sp[0] - ep[0])*(sp[0] - ep[0])
		+ (sp[1] - ep[1]) *(sp[1] - ep[1]));
		distall =distall + dist ;  
		return distall;

}
/******************************************************************
**	int set_ColorIndex()
**  
**	   
**
**  **avb
**
****************************/
int set_ColorIndex(const char *str)
{
	
	ads_point pn;
	ads_name en;
	AcDbObjectId eId;
	AcDbEntity* pEnt;
	int color;
    int rc = acedEntSel(str, en, pn);
	if (rc == RTCAN) 
		return color=500;
	if (rc == RTERROR )
		return color=300;

acdbGetObjectId(eId, en);
acdbOpenObject(pEnt, eId, AcDb::kForWrite);
color=pEnt->colorIndex();
if (color==256)
{
char* 	nameLayer=pEnt->layer();
pEnt->close();		
color=getLayerColor(nameLayer);

}
else
pEnt->close();
const char* s="\n   - %i";
acutPrintf(s,color);

		return color;
}
/****************************************************************************
**
**   
**
**  **avb
**
*************************************/

AcGePoint3d
midpoint(const AcGePoint3d& pt1, const AcGePoint3d& pt2)
{
    AcGePoint3d newPt;

    newPt.x =(pt1.x + pt2.x) / 2.0;
    newPt.y =(pt1.y + pt2.y) / 2.0;
    newPt.z =(pt1.z + pt2.z) / 2.0;

    return newPt;
}

/******************************************************************
**  void chSubReal()
**  
**	  Real  
**
**  **avb
**
****************************/
void chSubReal(ads_name en, int dxfType ,double nl)
{
ads_name sen; 
struct resbuf *ed, *cb; 
acdbEntNext(en, sen); 
if ((ed = acdbEntGet(sen)) != NULL) { 
    for (cb = ed; cb != NULL; cb = cb->rbnext) 
        if (cb->restype == dxfType) { // Start point DXF code
            cb->resval.rreal = nl;// Change coordinates.
           if (acdbEntMod(ed) != RTNORM) { // Move vertex.
                acutRelRb(ed); 
                return ; }
				else { 
               acdbEntUpd(en); // Regen the polyline.
                acutRelRb(ed); 
                return ; // Indicate success.
           } 
        } 
    acutRelRb(ed); 
} 
return ; // Indicate failure.
}
/******************************************************************
**  void chReal()
**  
**	  Real 
**
**  **avb
**
****************************/
void chReal(ads_name en, int dxfType ,double nl)
{
struct resbuf *ed, *cb; 
 
if ((ed = acdbEntGet(en)) != NULL) { 
    for (cb = ed; cb != NULL; cb = cb->rbnext) 
 
        if (cb->restype == dxfType) { // Start point DXF code
            cb->resval.rreal = nl;// Change coordinates.
            
 
            if (acdbEntMod(ed) != RTNORM) { // Move vertex.

                acutRelRb(ed); 
                return ; } 
        } 
    acutRelRb(ed); 
} 

return ; // Indicate failure.
}

/******************************************************************
**	int printList(struct resbuf *pBuf)
**  
**	 ,    SDK.
**	  resbufs   ,
**	  
**
**  **avb
**
****************************/
int printList(struct resbuf *pBuf)
{
    int rt, i;
    char buf[133];     
    acutPrintf("\n");  
   
    for (i = 0;pBuf != NULL;i++, pBuf = pBuf->rbnext)
    {
        if (pBuf->restype < 0)
            rt = pBuf->restype;         
        else if (pBuf->restype < 10)
            rt = RTSTR;
        else if (pBuf->restype < 38)
            rt = RT3DPOINT;
        else if (pBuf->restype < 60)
            rt = RTREAL;
        else if (pBuf->restype < 80)
            rt = RTSHORT;
        else if (pBuf->restype < 100)
            rt = RTLONG;
        else if (pBuf->restype < 106)
            rt = RTSTR;
        else if (pBuf->restype < 148)
            rt = RTREAL;
        else if (pBuf->restype < 290)
            rt = RTSHORT;
        else if (pBuf->restype < 320)
            rt = RTSTR;
        else if (pBuf->restype < 370)
            rt = RTENAME;
        else if (pBuf->restype < 999)
            rt = RT3DPOINT;
        else                      // pBuf->restype is already RTSHORT,
            rt = pBuf->restype;   // RTSTR, etc. or else it is unknown.

        switch (rt) {
        case RTSHORT:
            if (pBuf->restype == RTSHORT)
                acutPrintf("RTSHORT : %d\n", pBuf->resval.rint);
            else
                acutPrintf("(%d . %d)\n", pBuf->restype, pBuf->resval.rint);
            break;
        case RTREAL:
            if (pBuf->restype == RTREAL)
                acutPrintf("RTREAL : %0.3f\n", pBuf->resval.rreal);
            else
                acutPrintf("(%d . %0.3f)\n", pBuf->restype
                    , pBuf->resval.rreal);
            break;
        case RTSTR:
            if (pBuf->restype == RTSTR)
                acutPrintf("RTSTR : %s\n", pBuf->resval.rstring);
            else
                acutPrintf("(%d . \"%s\")\n", pBuf->restype
                    , pBuf->resval.rstring);
            break;
        case RT3DPOINT:
            if (pBuf->restype == RT3DPOINT)
                acutPrintf("RT3DPOINT : %0.3f, %0.3f, %0.3f\n",
                    pBuf->resval.rpoint[X],  pBuf->resval.rpoint[Y]
                    , pBuf->resval.rpoint[Z]);
            else
                acutPrintf("(%d %0.3f %0.3f %0.3f)\n", pBuf->restype,
                    pBuf->resval.rpoint[X], pBuf->resval.rpoint[Y],
                    pBuf->resval.rpoint[Z]);
            break;
        case RTLONG:
            acutPrintf("RTLONG : %dl\n", pBuf->resval.rlong);
            break;
        case -1:
        case RTENAME:                          // First block entity
            acutPrintf("(%d . <Entity name: %8lx>)\n", pBuf->restype,
                pBuf->resval.rlname[0]);
            break;
        case -3:                              // marks start of xdata
            acutPrintf("(-3)\n");
        }

        if ((i == 23) && (pBuf->rbnext != NULL)) {
            i = 0;
            acedGetString(0,"Press <ENTER> to continue...", buf);
        }
    }
    return RTNORM;
}
/************************************************************************
**
**  dxfCodeToDataType:
**
**  **avb
**
******************************************/

int
dxfCodeToDataType(int resType)
{
        // which data type is this value
    if ((resType >= 0) && (resType <= 9))
        return RTSTR;
    else if ((resType >= 10) && (resType <= 17))
        return RT3DPOINT;
    else if ((resType >= 38) && (resType <= 59))
        return RTREAL;
    else if ((resType >= 60) && (resType <= 79))
        return RTSHORT;
    else if ((resType >= 90) && (resType <= 99))
        return RTLONG;
    else if ((resType == 100) || (resType == 101) || (resType == 102) ||(resType == 105))
        return RTSTR;
	else if ((resType >= 110) && (resType <= 112))
		return RTSTR;
    else if ((resType >= 140) && (resType <= 149))
        return RTREAL;
    else if ((resType >= 170) && (resType <= 179))
        return RTSHORT;
    else if ((resType >= 210) && (resType <= 219))
        return RT3DPOINT;
    else if ((resType >= 270) && (resType <= 299))
        return RTSHORT;
    else if ((resType >= 300) && (resType <= 309))
        return RTSTR;
    else if ((resType >= 310) && (resType <= 369))
        return RTENAME;
    else if ((resType >= 370) && (resType <= 379))
        return RTSHORT;
    else if ((resType >= 380) && (resType <= 389))
        return RTSHORT;
    else if ((resType >= 390) && (resType <= 399))
        return RTENAME;
    else if ((resType >= 400) && (resType <= 409))
        return RTSHORT;
    else if ((resType >= 410) && (resType <= 419))
        return RTSTR;
    else if (resType == 1004)
        return resType;        // binary chunk
    else if ((resType >= 999) && (resType <= 1009))
        return RTSTR;
    else if ((resType >= 1010) && (resType <= 1013))
        return RT3DPOINT;
    else if ((resType >= 1038) && (resType <= 1059))
        return RTREAL;
    else if ((resType >= 1060) && (resType <= 1070))
        return RTSHORT;
    else if ((resType == 1071))
        return RTLONG;
    else if ((resType < 0) || (resType > 4999))
        return resType;
    else
        return RTNONE;
}